[data-md-color-scheme="jamsuite"]
{
--brand-red: #C9151B;
--bolt-yellow: #FFD21E;
--bg-primary: #F7F2EC; /* # EEDCC8; */
--bg-secondary: #FFF;
--bg-thirdiary: #EEDCC8;
--text-primary: #141414;
--text-secondary: #5F6A72;
--text-link-hover: #2E6FAE;
--text-link: #245A8C;
--md-primary-fg-color: var(--bg-secondary);
--md-primary-fg-color--light:#ECB7B7;
--md-primary-fg-color--dark: #90030C;
--md-default-bg-color: var(--bg-primary);
--md-typeset-a-color: var(--text-link);
--md-accent-fg-color: var(--text-link-hover);
--md-code-bg-color: #FAF7F3;
}

[data-md-color-scheme="jamsuite"]
.md-header {
    color: var(--text-primary);
}

[data-md-color-scheme="jamsuite"]
.md-header__title {
    margin-left: 0;
    font-family: 'Poppins', sans-serif;
    font-weight: 700;
}

[data-md-color-scheme="jamsuite"]
h1,h2,h3,h4,h5,h6 {
    font-family: 'Poppins', sans-serif;
}

[data-md-color-scheme="jamsuite"]
.md-content {
    background-color: var(--bg-secondary);
    border-radius: 7px;
    margin-bottom: 2rem;
    box-shadow: 0 .2rem .4rem #0000001a;
}

[data-md-color-scheme="jamsuite"]
.md-search__form {
    background-color: var(--bg-primary);
    border-radius: 2rem;
}

[data-md-color-scheme="jamsuite"]
.md-search__form:hover {
    background-color: var(--bg-primary);
}

[data-md-color-scheme="jamsuite"]
.md-search__input+.md-search__icon {
    color: var(--md-default-fg-color--light);
}

[data-md-color-scheme="jamsuite"]
.md-search__input {
    color: var(--text-primary);
}

[data-md-color-scheme="jamsuite"]
.md-search__input::placeholder {
    color: var(--md-default-fg-color--light);
}

